
import os
import logging
import cartopy
from WCD_shared_composites import compute_composites, plot_clustering, plot_quiver_clustering
import argparse
import yaml

logger = logging.getLogger(__name__)
FORMAT = "%(asctime)-15s %(message)s"
logging.basicConfig(format=FORMAT)
logging.getLogger().setLevel(logging.INFO)


os.environ["BSC_MACHINE"]

if 'BSC_MACHINE' in os.environ and os.environ['BSC_MACHINE'] == 'mn5':
    logger.info('Setting cartopy config for {} machine'.format(os.environ['BSC_MACHINE']))
    cartopy.config['data_dir'] = '/gpfs/projects/bsc32/bsc032259/data/cartopy/'

current_dir = os.path.dirname(os.path.abspath(__file__))


def start_compute_composites(config):

    ks = config["ks"]
    period = config["period"]
    bounds = config["bounds"] 
    minimum_intrusion_points_pcnt = config["minimum_intrusion_points_pcnt"] 
    sahara_clim_roll = config["sahara_clim_roll"]
    sahara_region = config["sahara_region"] 
    ta_inf = config["ta_inf"]
    ta_sup = config["ta_sup"]
    g_inf = config["g_inf"]
    g_sup = config["g_sup"]
    cluster_vars = config["cluster_vars"]

    ta_levels=f"{ta_sup.split('ta')[-1]}-{ta_inf.split('ta')[-1]}"
    gh_levels=f"{g_sup.split('g')[-1]}-{g_inf.split('g')[-1]}"

    region = config["region"]
    var_name = config["var_name"]
    level = config["level"]
    precursor_day = config["precursor_day"]

    if level == "None":
        level = None
    if region == "None":
        region = None

    quiver_level = config["quiver_level"]
    quiver_var_name = config["quiver_var_name"]
    composite_seasons = config["composite_seasons"]

    identifier=""

    for season in composite_seasons:
        k=ks[season]
        # for precursor_day in range(8):
        logger.info(var_name)
        logger.info(season)
        compute_composites(var_name, k, period, season, bounds, identifier, minimum_intrusion_points_pcnt, cluster_vars, region, sahara_clim_roll, sahara_region, ta_levels, gh_levels, precursor_day=precursor_day, level=level, var_from="", identification_season="ann", significance=True)
        plot_clustering(var_name, k, period, season, bounds, identifier, minimum_intrusion_points_pcnt, cluster_vars, region, sahara_clim_roll, sahara_region, ta_levels, gh_levels, precursor_day=precursor_day, level=level, var_from="", identification_season="ann", significance=True)

    logger.info("IF THE U AND V COMPOSITES FOR THE SPECIFIED quiver_level HAVEN'T BEEN COMPUTED THE QUIVER FUNCTION WILL FAIL")
    for season in composite_seasons:
        k=ks[season]
        # for precursor_day in range(8):
        logger.info(quiver_var_name)
        logger.info(season)
        plot_quiver_clustering(quiver_var_name, quiver_level, k, period, season, bounds, identifier, minimum_intrusion_points_pcnt, cluster_vars, region, sahara_clim_roll, sahara_region, ta_levels, gh_levels, precursor_day=0, var_from="", identification_season="ann", significance=True)


if __name__ == '__main__':

    with run_diagnostic() as config:
        start_compute_composites(config)
