import iris
import numpy as np
import os
import xarray as xr
import logging
import cartopy
from esmvalcore.preprocessor import regrid_time, extract_levels
from WCD_shared import episodes_ploter, dates_file_writer, pseudof_study, taylored_clustering, compute_continuous_intrusion_dates, extract_months_xarray, compute_continuous_thresholds
import yaml
from esmvaltool.diag_scripts.shared  import run_diagnostic, group_metadata
import argparse
import cf_units

logger = logging.getLogger(__name__)
FORMAT = "%(asctime)-15s %(message)s"
logging.basicConfig(format=FORMAT)
logging.getLogger().setLevel(logging.INFO)

os.environ["BSC_MACHINE"]

if 'BSC_MACHINE' in os.environ and os.environ['BSC_MACHINE'] == 'mn5':
    logger.info('Setting cartopy config for {} machine'.format(os.environ['BSC_MACHINE']))
    cartopy.config['data_dir'] = '/gpfs/projects/bsc32/bsc032259/data/cartopy/'

current_dir = os.path.dirname(os.path.abspath(__file__))



def find_intrusions(esmval_cfg):
    
    ks = esmval_cfg["ks"]
    identification_season = esmval_cfg["identification_season"] 
    period = esmval_cfg["period"]
    bounds = esmval_cfg["bounds"] 
    minimum_intrusion_points_pcnt = esmval_cfg["minimum_intrusion_points_pcnt"] 
    sahara_clim_roll = esmval_cfg["sahara_clim_roll"]
    sahara_region = esmval_cfg["sahara_region"] 
    ta_inf = esmval_cfg["ta_inf"]
    ta_sup = esmval_cfg["ta_sup"]
    g_inf = esmval_cfg["g_inf"]
    g_sup = esmval_cfg["g_sup"]
    cluster_vars = esmval_cfg["cluster_vars"]

    identifier = ""
    ilon,flon,ilat,flat = bounds
    ta_levels=f"{ta_sup.split('ta')[-1]}-{ta_inf.split('ta')[-1]}"
    gh_levels=f"{g_sup.split('g')[-1]}-{g_inf.split('g')[-1]}"

    indicators=f"{ta_sup}-{ta_inf}_{g_sup}-{g_inf}"
    logger.info(indicators)
    seasons_dict={i: s for i, s in zip(["djf", "mam", "jja", "son", "ann", "amjja", "mjj"],["a", "b", "c", "d","z","jj1","jj2"])}

    files_dict = {"theta": f"files/merged/formatted/avgtap_{ta_levels}_1959-2022_01-12.nc",
                  "thick": f"files/merged/formatted/gh_{gh_levels}_1959-2022_01-12.nc"}
    input_data=esmval_cfg['input_data'].values()
    alias_dict = group_metadata(input_data, "alias")
    for dataset in alias_dict:
        logger.info(dataset)
        if "MIROC" not in dataset:
            continue
        var_dict=group_metadata(alias_dict[dataset], "short_name")
        for var in var_dict:
            logger.info(var)
            build_indicator_files(dataset,var,var_dict,esmval_cfg)
        compute_continuous_thresholds(dataset, alias_dict, esmval_cfg, sahara_clim_roll, sahara_region, ta_levels, gh_levels)

        for season in ["ann", "djf", "mam", "jja", "son"]:
            k=ks[season]
            logger.info(f"computing {season}")
            dates_dataset = compute_continuous_intrusion_dates(dataset, esmval_cfg, var_dict, period, identification_season, bounds, identifier, minimum_intrusion_points_pcnt, sahara_clim_roll, sahara_region, ta_levels, gh_levels)
            print(1)
            desired_months = np.arange(1,13)
            if isinstance(season, str) and season != "ann":
                allmonths = "jfmamjjasond" * 2
                sstart = allmonths.index(season)
                desired_months = [i for i in range(sstart+1,sstart+1+len(season))]
            elif season == "ann":
                desired_months = np.arange(1,13)
            elif isinstance(season, int) or isinstance(season, np.int64):
                desired_months = [season]
            elif isinstance(season, int) or isinstance(season, np.int64):
                desired_months = [season]

            logger.info(bounds)
            ind_array=[]
            for cluster_var in cluster_vars:
                logger.info(cluster_var)
                file = os.path.join(esmval_cfg["work_dir"],var_dict[cluster_var][0]["filename"].split("/")[-1].replace("ta","avgtap").replace("zg", "gh"))
                dates_file_writer(dataset, file, esmval_cfg, cluster_var, dates_dataset, identifier, period, season, bounds, minimum_intrusion_points_pcnt, sahara_clim_roll, sahara_region, ta_levels, gh_levels)
                _p = xr.open_dataarray(os.path.join(esmval_cfg["work_dir"],f"{cluster_var}_intrusion_days",f"{dataset}_{cluster_var}_{ilon}.{flon}.{ilat}.{flat}_intrusions-clim-dates{identifier}_{period.replace('-','').replace('.','-')}_{season}_min-intr-points-{minimum_intrusion_points_pcnt}_sahara-{sahara_region[0]}.{sahara_region[1]}.{sahara_region[2]}.{sahara_region[3]}-roll.{sahara_clim_roll}-thresh_ta{ta_levels}_gh{gh_levels}.nc"))
                _p = (_p-_p.mean(axis=0))/_p.std(axis=0)
                logger.info(f"xr{cluster_var}")
                _p = extract_months_xarray(_p, desired_months)
                ind_array.append(_p)
            pseudof_study(dataset, esmval_cfg, ind_array, period, season, bounds, identifier, minimum_intrusion_points_pcnt, indicators)
            taylored_clustering(dataset, esmval_cfg, var_dict, k,ind_array, cluster_vars, period, season, bounds, identifier, minimum_intrusion_points_pcnt, sahara_clim_roll, sahara_region, ta_levels, gh_levels)
            events_da = episodes_ploter(dataset, esmval_cfg, var_dict, k, period, season, bounds, identifier, minimum_intrusion_points_pcnt, sahara_clim_roll, sahara_region, ta_levels, gh_levels, indicators, seasons_dict, cluster_vars=cluster_vars)


def convert_to_365_day_calendar(cube):
    """
    Convert any calendar in an Iris Cube to a 365-day calendar and remove February 29th.

    Parameters:
        cube (iris.cube.Cube): Input cube with a time coordinate.

    Returns:
        iris.cube.Cube: Cube with time converted to 365-day calendar and Feb 29 removed.
    """
    if cube.coord("time").units.calendar == "365_day":
        return cube
    # Extract time coordinate
    time_coord = cube.coord('time')

    # Convert time points to datetime objects
    time_units = time_coord.units
    time_datetimes = time_units.num2date(time_coord.points)

    # Remove February 29th
    filtered_time_points = [t for t in time_datetimes if not (t.month == 2 and t.day == 29)]

    # Convert back to numeric time points using a 365-day calendar
    new_time_units = cf_units.Unit(time_units.origin, calendar='365_day')
    new_time_points = [new_time_units.date2num(t) for t in filtered_time_points]

    # Create new time coordinate
    new_time_coord = iris.coords.DimCoord(new_time_points, standard_name='time', units=new_time_units)

    # Create a new cube with updated time coordinate
    new_cube = cube.extract(iris.Constraint(time=lambda t: not (t.point.month == 2 and t.point.day == 29)))

    # Replace the time coordinate
    new_cube.remove_coord('time')
    new_cube.add_dim_coord(new_time_coord, cube.coord_dims('time')[0])

    return new_cube

def build_indicator_files(dataset,var,var_dict,cfg):

    cube = iris.load_cube(var_dict[var][0]["filename"])
    if var == "ta":
        ta_inf=extract_levels(cube, [85000.],"nearest")*(1000/850)**(2/7)
        ta_sup=extract_levels(cube, [70000.],"nearest")*(1000/700)**(2/7)
        out_cube = (ta_inf+ta_sup)/2
        out_cube = convert_to_365_day_calendar(out_cube)
        out_cube.long_name = "average potential temperature"
    elif var == "zg":
        cube = cube
        z_inf = extract_levels(cube, [100000.], "nearest") 
        z_sup = extract_levels(cube, [50000.], "nearest") 
        out_cube = z_sup-z_inf
        out_cube = convert_to_365_day_calendar(out_cube)
        out_cube.long_name = "geopotential thickness"
        out_cube.units="m"
    else:
        return 0
    iris.save(out_cube, os.path.join(cfg["work_dir"],var_dict[var][0]["filename"].split("/")[-1].replace("ta","avgtap").replace("zg", "gh")))


if __name__ == '__main__':
    
    # parser = argparse.ArgumentParser(description="")

    # DEFAULT_FILE_PATH = os.path.join(current_dir,"config_find_intrusions.yml")
    # parser.add_argument("--config_file", "-f", type=str, default=DEFAULT_FILE_PATH, help="Path to the config file")

    # args = parser.parse_args()

    # if not os.path.isfile(args.config_file):
    #     print(f"Error: The file '{args.config_file}' does not exist.")
    #     exit(1)

    with run_diagnostic() as config:
        find_intrusions(config)


######################################################
######################################################
######################################################
######################################################
######################################################

# import cdsapi

# dataset = "reanalysis-era5-pressure-levels"
# request = {
#     "product_type": ["reanalysis"],
#     "variable": [
#         "geopotential",
#         "temperature",
#         "u_component_of_wind",
#         "v_component_of_wind"
#     ],
#     "year": [
#         "1940", "1941", "1942",
#         "1943", "1944", "1945",
#         "1946", "1947", "1948",
#         "1949", "1950", "1951",
#         "1952", "1953", "1954",
#         "1955", "1956", "1957",
#         "1958", "1959", "1960",
#         "1961", "1962", "1963",
#         "1964", "1965", "1966",
#         "1967", "1968", "1969",
#         "1970", "1971", "1972",
#         "1973", "1974", "1975",
#         "1976", "1977", "1978",
#         "1979", "1980", "1981",
#         "1982", "1983", "1984",
#         "1985", "1986", "1987",
#         "1988", "1989", "1990",
#         "1991", "1992", "1993",
#         "1994", "1995", "1996",
#         "1997", "1998", "1999",
#         "2000", "2001", "2002",
#         "2003", "2004", "2005",
#         "2006", "2007", "2008",
#         "2009", "2010", "2011",
#         "2012", "2013", "2014",
#         "2015", "2016", "2017",
#         "2018", "2019", "2020",
#         "2021", "2022", "2023",
#         "2024"
#     ],
#     "month": [
#         "01", "02", "03",
#         "04", "05", "06",
#         "07", "08", "09",
#         "10", "11", "12"
#     ],
#     "day": [
#         "01", "02", "03",
#         "04", "05", "06",
#         "07", "08", "09",
#         "10", "11", "12",
#         "13", "14", "15",
#         "16", "17", "18",
#         "19", "20", "21",
#         "22", "23", "24",
#         "25", "26", "27",
#         "28", "29", "30",
#         "31"
#     ],
#     "time": [
#         "00:00", "01:00", "02:00",
#         "03:00", "04:00", "05:00",
#         "06:00", "07:00", "08:00",
#         "09:00", "10:00", "11:00",
#         "12:00", "13:00", "14:00",
#         "15:00", "16:00", "17:00",
#         "18:00", "19:00", "20:00",
#         "21:00", "22:00", "23:00"
#     ],
#     "pressure_level": [
#         "200", "500", "700",
#         "850", "925", "1000"
#     ],
#     "data_format": "grib",
#     "download_format": "zip",
#     "area": [90, -180, 15, 180]
# }

# client = cdsapi.Client()
# client.retrieve(dataset, request).download()


######################################################
######################################################
######################################################
######################################################
######################################################