import iris
import numpy as np
import os
import xarray as xr
import logging
import cartopy
from esmvalcore.preprocessor import regrid_time
from WCD_shared import episodes_ploter, dates_file_writer, pseudof_study, taylored_clustering, compute_continuous_intrusion_dates, extract_months_xarray, compute_continuous_thresholds
import yaml
import argparse

logger = logging.getLogger(__name__)
FORMAT = "%(asctime)-15s %(message)s"
logging.basicConfig(format=FORMAT)
logging.getLogger().setLevel(logging.INFO)

os.environ["BSC_MACHINE"]

if 'BSC_MACHINE' in os.environ and os.environ['BSC_MACHINE'] == 'mn5':
    logger.info('Setting cartopy config for {} machine'.format(os.environ['BSC_MACHINE']))
    cartopy.config['data_dir'] = '/gpfs/projects/bsc32/bsc032259/data/cartopy/'

current_dir = os.path.dirname(os.path.abspath(__file__))



def find_intrusions(cfg_path):

    with open(cfg_path, "r") as f:
        config = yaml.safe_load(f)
    
    ks = config["intrusion_finder"]["ks"]
    identification_season = config["intrusion_finder"]["identification_season"] 
    period = config["intrusion_finder"]["period"]
    bounds = config["intrusion_finder"]["bounds"] 
    minimum_intrusion_points_pcnt = config["intrusion_finder"]["minimum_intrusion_points_pcnt"] 
    sahara_clim_roll = config["intrusion_finder"]["sahara_clim_roll"]
    sahara_region = config["intrusion_finder"]["sahara_region"] 
    ta_inf = config["intrusion_finder"]["ta_inf"]
    ta_sup = config["intrusion_finder"]["ta_sup"]
    g_inf = config["intrusion_finder"]["g_inf"]
    g_sup = config["intrusion_finder"]["g_sup"]
    cluster_vars = config["intrusion_finder"]["cluster_vars"]

    identifier = ""
    ilon,flon,ilat,flat = bounds
    ta_levels=f"{ta_sup.split('ta')[-1]}-{ta_inf.split('ta')[-1]}"
    gh_levels=f"{g_sup.split('g')[-1]}-{g_inf.split('g')[-1]}"

    indicators=f"{ta_sup}-{ta_inf}_{g_sup}-{g_inf}"
    logger.info(indicators)
    seasons_dict={i: s for i, s in zip(["djf", "mam", "jja", "son", "ann", "amjja", "mjj"],["a", "b", "c", "d","z","jj1","jj2"])}

    files_dict = {"theta": f"files/merged/formatted/avgtap_{ta_levels}_1959-2022_01-12.nc",
                  "thick": f"files/merged/formatted/gh_{gh_levels}_1959-2022_01-12.nc"}

    format_pt_gh()
    build_indicator_files(ta_inf, ta_sup, g_inf, g_sup, ta_levels, gh_levels)

    if not os.path.isfile(os.path.join(current_dir,f"files/merged/formatted/avgtap_{ta_levels}_1959-2022_01-12_sahara-{sahara_region[0]}.{sahara_region[1]}.{sahara_region[2]}.{sahara_region[3]}-roll.{sahara_clim_roll}-thresholds.nc")) or not os.path.isfile(os.path.join(current_dir,f"files/merged/formatted/gh_{gh_levels}_1959-2022_01-12_sahara-{sahara_region[0]}.{sahara_region[1]}.{sahara_region[2]}.{sahara_region[3]}-roll.{sahara_clim_roll}-thresholds.nc")):
        compute_continuous_thresholds(sahara_clim_roll, sahara_region, ta_levels, gh_levels)

    for season in ["ann", "djf", "mam", "jja", "son"]:
        k=ks[season]
        logger.info(f"computing {season}")
        dates_dataset = compute_continuous_intrusion_dates(period, identification_season, bounds, identifier, minimum_intrusion_points_pcnt, sahara_clim_roll, sahara_region, ta_levels, gh_levels)

        desired_months = np.arange(1,13)
        if isinstance(season, str) and season != "ann":
            allmonths = "jfmamjjasond" * 2
            sstart = allmonths.index(season)
            desired_months = [i for i in range(sstart+1,sstart+1+len(season))]
        elif season == "ann":
            desired_months = np.arange(1,13)
        elif isinstance(season, int) or isinstance(season, np.int64):
            desired_months = [season]
        elif isinstance(season, int) or isinstance(season, np.int64):
            desired_months = [season]

        logger.info(bounds)
        ind_array=[]
        for cluster_var in cluster_vars:
            file = os.path.join(current_dir, files_dict[cluster_var])
            dates_file_writer(file, cluster_var, dates_dataset, identifier, period, season, bounds, minimum_intrusion_points_pcnt, sahara_clim_roll, sahara_region, ta_levels, gh_levels)
            _p = xr.open_dataarray(os.path.join(current_dir,f"files/postpro_files/{cluster_var}_intrusion_days/{cluster_var}_{ilon}.{flon}.{ilat}.{flat}_intrusions-clim-dates{identifier}_{period.replace('-','').replace('.','-')}_{identification_season}_min-intr-points-{minimum_intrusion_points_pcnt}_sahara-{sahara_region[0]}.{sahara_region[1]}.{sahara_region[2]}.{sahara_region[3]}-roll.{sahara_clim_roll}-thresh_ta{ta_levels}_gh{gh_levels}.nc"))
            _p = (_p-_p.mean(axis=0))/_p.std(axis=0)
            logger.info(f"xr{cluster_var}")
            _p = extract_months_xarray(_p, desired_months)
            ind_array.append(_p)
        pseudof_study(ind_array, period, season, bounds, identifier, minimum_intrusion_points_pcnt, indicators)
        taylored_clustering(k,ind_array, cluster_vars, period, season, bounds, identifier, minimum_intrusion_points_pcnt, sahara_clim_roll, sahara_region, ta_levels, gh_levels)
        events_da = episodes_ploter(k, period, season, bounds, identifier, minimum_intrusion_points_pcnt, sahara_clim_roll, sahara_region, ta_levels, gh_levels, indicators, seasons_dict, cluster_vars=cluster_vars)

def build_indicator_files(ta_inf, ta_sup, g_inf, g_sup, ta_levels, gh_levels):
    file_ta_inf = os.path.join(current_dir,f"files/merged/formatted/{ta_inf}_1959-2022_01-12.nc")
    file_ta_sup = os.path.join(current_dir,f"files/merged/formatted/{ta_sup}_1959-2022_01-12.nc")
    file_g_inf = os.path.join(current_dir,f"files/merged/formatted/{g_inf}_1959-2022_01-12.nc")
    file_g_sup = os.path.join(current_dir,f"files/merged/formatted/{g_sup}_1959-2022_01-12.nc")

    ta_inf_da = iris.load_cube(file_ta_inf)
    ta_sup_da = iris.load_cube(file_ta_sup)
    ta = (ta_inf_da+ta_sup_da)/2
    ta.long_name = f"potential_temperature_avg_{ta_levels}"
    iris.save(ta, os.path.join(current_dir,f"files/merged/formatted/avgtap_{ta_levels}_1959-2022_01-12.nc"))
    g_inf_da = iris.load_cube(file_g_inf)
    g_sup_da = iris.load_cube(file_g_sup)
    g = g_sup_da-g_inf_da
    g.long_name = f"geopotential_thickness_{gh_levels}"
    iris.save(g, os.path.join(current_dir,f"files/merged/formatted/gh_{gh_levels}_1959-2022_01-12.nc"))

def format_pt_gh():
    path = os.path.join(current_dir,"files/merged")
    for var in ["g500", "g850", "g1000", "ta925", "ta850","ta700"]:
        file = os.path.join(path,f"{var}_1959-2022_01-12.nc")
        da = xr.open_dataset(file)
        da=da[list(da.var())[0]]

        if "plev" in da.coords:
            da = da.drop_vars(["plev"])
        if "dim_0" in da.dims:
            da = da.drop_vars(["dim_0"])
        if "ta" in var:
            da = da*(1000/int(da.name.split("ta")[-1]))**(2/7)
        elif "g" in var:
            da = da/9.80665
        da.name=f"formatted_{da.name}"
        da_iris = da.to_iris()
        logger.info(da_iris.coord("time"))
        da_iris=regrid_time(da_iris, "day")
        da_iris.coord("time").points = da_iris.coord("time").points+12
        da_iris.coord("time").bounds = None
        da_iris.coord("time").guess_bounds()
        da_iris.remove_coord("day_of_month")
        da_iris.remove_coord("day_of_year")
        logger.info(da_iris.coord("time"))
        iris.save(da_iris,os.path.join(path,f"formatted/{var}_1959-2022_01-12.nc"))
    

if __name__ == '__main__':
    
    parser = argparse.ArgumentParser(description="")

    DEFAULT_FILE_PATH = os.path.join(current_dir,"config_find_intrusions.yml")
    parser.add_argument("--config_file", "-f", type=str, default=DEFAULT_FILE_PATH, help="Path to the config file")

    args = parser.parse_args()

    if not os.path.isfile(args.config_file):
        print(f"Error: The file '{args.config_file}' does not exist.")
        exit(1)

    find_intrusions(args.config_file)


######################################################
######################################################
######################################################
######################################################
######################################################

# import cdsapi

# dataset = "reanalysis-era5-pressure-levels"
# request = {
#     "product_type": ["reanalysis"],
#     "variable": [
#         "geopotential",
#         "temperature",
#         "u_component_of_wind",
#         "v_component_of_wind"
#     ],
#     "year": [
#         "1940", "1941", "1942",
#         "1943", "1944", "1945",
#         "1946", "1947", "1948",
#         "1949", "1950", "1951",
#         "1952", "1953", "1954",
#         "1955", "1956", "1957",
#         "1958", "1959", "1960",
#         "1961", "1962", "1963",
#         "1964", "1965", "1966",
#         "1967", "1968", "1969",
#         "1970", "1971", "1972",
#         "1973", "1974", "1975",
#         "1976", "1977", "1978",
#         "1979", "1980", "1981",
#         "1982", "1983", "1984",
#         "1985", "1986", "1987",
#         "1988", "1989", "1990",
#         "1991", "1992", "1993",
#         "1994", "1995", "1996",
#         "1997", "1998", "1999",
#         "2000", "2001", "2002",
#         "2003", "2004", "2005",
#         "2006", "2007", "2008",
#         "2009", "2010", "2011",
#         "2012", "2013", "2014",
#         "2015", "2016", "2017",
#         "2018", "2019", "2020",
#         "2021", "2022", "2023",
#         "2024"
#     ],
#     "month": [
#         "01", "02", "03",
#         "04", "05", "06",
#         "07", "08", "09",
#         "10", "11", "12"
#     ],
#     "day": [
#         "01", "02", "03",
#         "04", "05", "06",
#         "07", "08", "09",
#         "10", "11", "12",
#         "13", "14", "15",
#         "16", "17", "18",
#         "19", "20", "21",
#         "22", "23", "24",
#         "25", "26", "27",
#         "28", "29", "30",
#         "31"
#     ],
#     "time": [
#         "00:00", "01:00", "02:00",
#         "03:00", "04:00", "05:00",
#         "06:00", "07:00", "08:00",
#         "09:00", "10:00", "11:00",
#         "12:00", "13:00", "14:00",
#         "15:00", "16:00", "17:00",
#         "18:00", "19:00", "20:00",
#         "21:00", "22:00", "23:00"
#     ],
#     "pressure_level": [
#         "200", "500", "700",
#         "850", "925", "1000"
#     ],
#     "data_format": "grib",
#     "download_format": "zip",
#     "area": [90, -180, 15, 180]
# }

# client = cdsapi.Client()
# client.retrieve(dataset, request).download()


######################################################
######################################################
######################################################
######################################################
######################################################