import iris
import matplotlib.pyplot as plt
import cartopy.crs as ccrs
import numpy as np
import os
import xarray as xr
import logging
import cartopy
import pandas as pd
from esmvalcore.preprocessor import extract_month, extract_time, extract_season
from WCD_shared import plot_area_of_impact
from matplotlib.colors import BoundaryNorm, ListedColormap


logger = logging.getLogger(__name__)
FORMAT = "%(asctime)-15s %(message)s"
logging.basicConfig(format=FORMAT)
logging.getLogger().setLevel(logging.INFO)

os.environ["BSC_MACHINE"]

if 'BSC_MACHINE' in os.environ and os.environ['BSC_MACHINE'] == 'mn5':
    logger.info('Setting cartopy config for {} machine'.format(os.environ['BSC_MACHINE']))
    cartopy.config['data_dir'] = '/gpfs/projects/bsc32/bsc032259/data/cartopy/'

current_dir = os.path.dirname(os.path.abspath(__file__))

extent=[-25, 50, 15, 60]

# COLORBARS
levels100 = np.linspace(0, 100, 11)  # 10 intervals from 0 to 100
# Create a discrete colormap with BoundaryNorm
cmap100 = plt.get_cmap("Oranges", len(levels100) - 1)  # Colormap with 10 discrete colors
colors = cmap100(np.arange(cmap100.N))
colors[0] = [1, 1, 1, 1]  # White in RGBA
custom_cmap100 = plt.cm.colors.ListedColormap(colors)
# Create a BoundaryNorm for the levels
norm100 = BoundaryNorm(levels100, ncolors=custom_cmap100.N, clip=True)

levels12 = np.linspace(0, 12, 13)  # 10 intervals from 0 to 100
# Create a discrete colormap with BoundaryNorm
cmap12 = plt.get_cmap("Oranges", len(levels12) - 1)  # Colormap with 10 discrete colors
norm12 = BoundaryNorm(levels12, ncolors=cmap12.N, clip=True)

levels20 = np.linspace(8, 12, 13)  # 10 intervals from 0 to 100
# Create a discrete colormap with BoundaryNorm
cmap20 = plt.get_cmap("Oranges", len(levels20) - 1)  # Colormap with 10 discrete colors
norm20 = BoundaryNorm(levels20, ncolors=cmap20.N, clip=True)

levelsn = np.linspace(-3, 3, 13)  # 10 intervals from 0 to 100
cmapn = plt.get_cmap("RdBu_r", len(levelsn) - 1)  # Colormap with 10 discrete colors
normn = BoundaryNorm(levelsn, ncolors=cmapn.N, clip=True)

# levelsnL = np.linspace(-4, 6, 13)  # 10 intervals from 0 to 100
# cmapnL = plt.get_cmap("RdBu_r", len(levelsnL) - 1)  # Colormap with 10 discrete colors
# normnL = BoundaryNorm(levelsnL, ncolors=cmapnL.N, clip=True)
# cmapnL = plt.get_cmap("RdBu_r", len(levelsnL) - 1)
# # Define the norm with a transition at 1
# Define levels for the colorbar
# levelsnL = np.linspace(0, 10, 13)  # 13 evenly spaced intervals from 0 to 6
# Create the RdBu_r colormap
levelsnL = np.array(list(np.arange(0, 1, .1))+list(np.arange(1,12,1)))  # 13 evenly spaced intervals from 0 to 6
cmapnL = plt.get_cmap("RdBu_r", len(levelsnL) - 1)
# Convert to a ListedColormap to modify colors
new_colors = cmapnL(np.arange(cmapnL.N))  # Get colors as an array
new_colors[0] = [1, 1, 1, 1]  # Set first color to white (RGBA: [1, 1, 1, 1])
# Create new colormap
new_cmapnL = ListedColormap(new_colors)
normnL = BoundaryNorm(levelsnL, ncolors=new_cmapnL.N)# , clip=True)
# normnL = TwoSlopeNorm(vmin=0.1, vcenter=1, vmax=10)
# normnL = LogNorm(vmin=0.01, vmax=10)  # LogNorm handles 0 by starting slightly above it


def find_pX_days(season, period, X):
    fs = [[os.path.join(current_dir, f"files/tasmax_1959-2022_01-12.nc"), os.path.join(current_dir, f"files/tasmax-p{X}-7dayrolling_{period}.nc")],
          [os.path.join(current_dir, f"files/stations.nc"),os.path.join(current_dir, f"files/station-tasmax-p{X}-7dayrolling_{period}.nc")]]
    pXs = []
    for f in fs:
        ss,es=period.split(".")
        sy,sm,sd = ss.split("-")
        ey,em,ed = es.split("-")
        if "stations" in f[0]:
            tasmax = extract_time(extract_season(xr.open_dataarray(f[0]).to_iris(),season),sy,sm,sd,ey,em,ed)
        else:
            tasmax = extract_time(extract_season(xr.open_dataset(f[0]).tasmax.to_iris(),season),sy,sm,sd,ey,em,ed)
        pX = extract_season(iris.load_cube(f[1]),season)

        if isinstance(season, str):
            tasmax = extract_season(tasmax, season)
            pX = extract_season(pX, season)
        elif isinstance(season, int):
            tasmax = extract_month(tasmax, season)
            pX = extract_month(pX, season)
        # intrusion dates
        pX_days = xr.DataArray.from_iris(tasmax) >= xr.DataArray.from_iris(pX)
        pX_days = pX_days.where(pX_days)
        pX_days_time_values = [str(i).split("T")[0] for i in pX_days.time.values]
        pX_days = pX_days.assign_coords(time=pX_days_time_values)    
        pX_days.to_netcdf(f[1].replace(".nc", f"_{season}.nc"))
        pXs.append(pX_days)
    return pXs

# P(intrusion|extreme t)
def plot_intrusion_cond_pX(cluster_vars, pX_days, pX_days_station, k, period, period_cluster, bounds, minimum_intrusion_points_pcnt, identifier, season, row_id, fig, gs, sahara_clim_roll, sahara_region, percentile, ta_levels, gh_levels):
    ilon,flon,ilat,flat = bounds
    ss,es=period.split(".")
    sy,sm,sd = ss.split("-")
    ey,em,ed = es.split("-")
    clusters_vector_file = f"{current_dir}/files/postpro_files/clusters-k{k}_{'.'.join(cluster_vars)}_{ilon}.{flon}.{ilat}.{flat}_intrusions-clim-dates{identifier}_{period_cluster.replace('-','').replace('.','-')}_{season}_min-intr-points-{minimum_intrusion_points_pcnt}_sahara-{sahara_region[0]}.{sahara_region[1]}.{sahara_region[2]}.{sahara_region[3]}-roll.{sahara_clim_roll}-thresh_ta{ta_levels}_gh{gh_levels}.nc"
    clusters = xr.open_dataarray(clusters_vector_file).sel(time=slice(sy,ey))

    if str(percentile) == "90":
        label=r"P(IT\textit{n} intrusion day $\mid$ TX90p) (%)"
    elif str(percentile) == "99":
        label=r"P(IT\textit{n} intrusion day $\mid$ TX99p) (%)"

    percent_of_intrusion_days_coinciding_withpX = []
    for cluster_n in range(k):
        cluster_type_time = clusters.where(clusters==cluster_n, drop=True).time
        dates=pX_days.time
        is_in_dates = np.isin(dates, cluster_type_time)
        counts=np.count_nonzero(~np.isnan(pX_days[is_in_dates,:,:]),axis=0)/np.count_nonzero(~np.isnan(pX_days),axis=0)*100
        dates_station=pX_days_station.time
        is_in_dates = np.isin(dates_station, cluster_type_time)
        counts_station = np.count_nonzero(~np.isnan(pX_days_station[is_in_dates,:]),axis=0)/np.count_nonzero(~np.isnan(pX_days_station),axis=0)*100
        coord = {"Cluster_N": ("time",np.ones(pX_days.shape[0])+cluster_n)}
        logger.info(f"intrusion days for cluster {cluster_n+1}: {cluster_type_time.shape[0]}")
        ax = fig.add_subplot(gs[row_id, cluster_n], projection=ccrs.PlateCarree())
        img = pX_days.drop_vars(["season_year"]).assign_coords(coord)[0].drop_vars(["time"]).copy(data=counts).rename(f"percentage of p{percentile} days coinciding with intrusion").plot(vmin=0, vmax=12, cmap="Oranges", add_colorbar=False)
        ax.coastlines(alpha=0.7)
        ax.scatter(
            [float(i.split(":")[-1])for i in pX_days_station.location.values],
            [float(i.split(":")[0])for i in pX_days_station.location.values],
            c=counts_station,
            s=45, 
            cmap="Oranges",
            edgecolor="black",
            linewidth=0.8,
            vmin=0,
            vmax=12,
            zorder=100,
        )
        ax.set_extent(extent)
    ss,es=period.split(".")
    sy,sm,sd = ss.split("-")
    ey,em,ed = es.split("-")
    clusters_vector_file = f"{current_dir}/files/postpro_files/clusters-k{k}_{'.'.join(cluster_vars)}_{ilon}.{flon}.{ilat}.{flat}_intrusions-clim-dates{identifier}_{period_cluster.replace('-','').replace('.','-')}_{season}_min-intr-points-{minimum_intrusion_points_pcnt}_sahara-{sahara_region[0]}.{sahara_region[1]}.{sahara_region[2]}.{sahara_region[3]}-roll.{sahara_clim_roll}-thresh_ta{ta_levels}_gh{gh_levels}.nc"
    clusters = xr.open_dataarray(clusters_vector_file).sel(time=slice(sy,ey))

    percent_of_intrusion_days_coinciding_withpX = []
    for cluster_n in range(k):
        cluster_type_time = clusters.where(clusters==cluster_n, drop=True).time
        dates=pX_days.time
        is_in_dates = np.isin(dates, cluster_type_time)
        counts=np.count_nonzero(~np.isnan(pX_days[is_in_dates,:,:]),axis=0)/cluster_type_time.shape[0]*100
        dates_station=pX_days_station.time
        is_in_dates = np.isin(dates_station, cluster_type_time)
        counts_station = np.count_nonzero(~np.isnan(pX_days_station[is_in_dates,:]),axis=0)/cluster_type_time.shape[0]*100
        coord = {"Cluster_N": ("time",np.ones(pX_days.shape[0])+cluster_n)}
        logger.info(f"intrusion days for cluster {cluster_n+1}: {cluster_type_time.shape[0]}")
        ax = fig.add_subplot(gs[row_id, cluster_n], projection=ccrs.PlateCarree())
        dsplot = pX_days.drop_vars(["season_year"]).assign_coords(coord)[0].drop_vars(["time"]).copy(data=counts).rename(f"percentage of p{percentile} days coinciding with intrusion")
        dsplot.to_netcdf(f"{current_dir}/files/p{percentile}/p{percentile}-cond-intrusion_clusters-k{k}_{'.'.join(cluster_vars)}_{ilon}.{flon}.{ilat}.{flat}_intrusions-clim-dates{identifier}_{period_cluster.replace('-','').replace('.','-')}_{season}_min-intr-points-{minimum_intrusion_points_pcnt}_sahara-{sahara_region[0]}.{sahara_region[1]}.{sahara_region[2]}.{sahara_region[3]}-roll.{sahara_clim_roll}-thresh_ta{ta_levels}_gh{gh_levels}.nc")
        img = dsplot.plot(norm=norm12, cmap=cmap12, add_colorbar=False)
        ax.coastlines(alpha=0.7)
        ax.scatter(
            [float(i.split(":")[-1])for i in pX_days_station.location.values],
            [float(i.split(":")[0])for i in pX_days_station.location.values],
            c=counts_station,
            s=45,
            cmap=cmap12,
            edgecolor="black",
            linewidth=0.8,
            norm=norm12,
            zorder=100,
        )
        ax.set_extent(extent)
        area_of_impact = xr.open_dataarray(os.path.join(current_dir,f"files/postpro_files/area-of-impact-n{cluster_n+1}_clusters-k{k}_{'.'.join(cluster_vars)}_{ilon}.{flon}.{ilat}.{flat}_intrusions-clim-dates{identifier}_{period_cluster.replace('-','').replace('.','-')}_{season}_min-intr-points-{minimum_intrusion_points_pcnt}_sahara-{sahara_region[0]}.{sahara_region[1]}.{sahara_region[2]}.{sahara_region[3]}-roll.{sahara_clim_roll}-thresh_ta{ta_levels}_gh{gh_levels}.nc"))
        ax = plot_area_of_impact(ax, area_of_impact, "solid")
        ax.set_title(f"{season.upper()} intrusion type {cluster_n+1}\n{cluster_type_time.shape[0]} intrusion days")
        percent_of_intrusion_days_coinciding_withpX.append(pX_days[0].copy(data=counts))

        if cluster_n*row_id == 9:
            # Add a new axis for the colorbar
            cbar_ax = fig.add_axes([0.3, 0.03, 0.4, 0.02])  # Adjust the position [left, bottom, width, height]
            # Add the colorbar to the new axis
            cbar = plt.colorbar(plt.cm.ScalarMappable(norm=norm12, cmap=cmap12), ticks=levels12, spacing='uniform', cax=cbar_ax, orientation='horizontal', label=label)
            # plt.colorbar(img, cax=cbar_ax, orientation='horizontal', label="percentage of pX days that are intrusion days*****")

# P(extreme t|no intru)
def plot_pX_cond_nointrusion(cluster_vars, pX_days, pX_days_station, k, period, period_cluster, bounds, minimum_intrusion_points_pcnt, identifier, season, row_id, fig, gs, sahara_clim_roll, sahara_region, percentile, ta_levels, gh_levels):
    ilon,flon,ilat,flat = bounds
    ss,es=period.split(".")
    sy,sm,sd = ss.split("-")
    ey,em,ed = es.split("-")
    clusters_vector_file = f"{current_dir}/files/postpro_files/clusters-k{k}_{'.'.join(cluster_vars)}_{ilon}.{flon}.{ilat}.{flat}_intrusions-clim-dates{identifier}_{period_cluster.replace('-','').replace('.','-')}_{season}_min-intr-points-{minimum_intrusion_points_pcnt}_sahara-{sahara_region[0]}.{sahara_region[1]}.{sahara_region[2]}.{sahara_region[3]}-roll.{sahara_clim_roll}-thresh_ta{ta_levels}_gh{gh_levels}.nc"
    clusters = xr.open_dataarray(clusters_vector_file).sel(time=slice(sy,ey))
    if str(percentile) == "90":
        label=r"P(TX90p $\mid$ no IT\textit{n} intrusion day)"
    elif str(percentile) == "99":
        label=r"P(TX99p $\mid$ no IT\textit{n} intrusion day)"
    percent_of_intrusion_days_coinciding_withpX = []
    intrusion_dates=clusters.time 
    # P(TX90|no intrusion) = P(no intru|TX90) * P(TX90) / P(no intru) = (No intrusion days in pX record) * 0.1 / (# no intrusion days / all days) 
    for cluster_n in range(k):
        cluster_type_time = clusters.where(clusters==cluster_n, drop=True).time
        dates=pX_days.time
        # The curl so that is not in dates
        # is_in_dates = np.isin(dates, clusters.time)
        is_in_dates = np.isin(dates, cluster_type_time.time)
        # counts=np.count_nonzero(~np.isnan(pX_days[~is_in_dates,:,:]),axis=0)/(dates.shape[0]-clusters.shape[0])*100
        counts=np.count_nonzero(~np.isnan(pX_days[~is_in_dates,:,:]),axis=0)/(dates.shape[0]-cluster_type_time.shape[0])*100#-10
        dates_station=pX_days_station.time
        is_in_dates = np.isin(dates_station, cluster_type_time)
        # counts_station = np.count_nonzero(~np.isnan(pX_days_station[~is_in_dates,:]),axis=0)/(dates_station.shape[0]-clusters.shape[0])*100 #-10
        counts_station = np.count_nonzero(~np.isnan(pX_days_station[~is_in_dates,:]),axis=0)/(dates_station.shape[0]-cluster_type_time.shape[0])*100 #-10
        coord = {"Cluster_N": ("time",np.ones(pX_days.shape[0])+cluster_n)}
        logger.info(f"intrusion days for cluster {cluster_n+1}: {cluster_type_time.shape[0]}")
        ax = fig.add_subplot(gs[row_id, cluster_n], projection=ccrs.PlateCarree())
        dsplot = pX_days.drop_vars(["season_year"]).assign_coords(coord)[0].drop_vars(["time"]).copy(data=counts).rename(f"percentage_of_p{percentile}_days_coinciding_with_intrusion")
        dsplot.to_netcdf(f"{current_dir}/files/p{percentile}/p{percentile}-cond-nointrusion_n{cluster_n}_clusters-k{k}_{'.'.join(cluster_vars)}_{ilon}.{flon}.{ilat}.{flat}_intrusions-clim-dates{identifier}_{period_cluster.replace('-','').replace('.','-')}_{season}_min-intr-points-{minimum_intrusion_points_pcnt}_sahara-{sahara_region[0]}.{sahara_region[1]}.{sahara_region[2]}.{sahara_region[3]}-roll.{sahara_clim_roll}-thresh_ta{ta_levels}_gh{gh_levels}.nc")
        station_nc = pX_days_station[0]
        station_nc.values = counts_station
        station_nc.to_netcdf(f"{current_dir}/files/p{percentile}/p{percentile}-cond-nointrusion_n{cluster_n}_station_clusters-k{k}_{'.'.join(cluster_vars)}_{ilon}.{flon}.{ilat}.{flat}_intrusions-clim-dates{identifier}_{period_cluster.replace('-','').replace('.','-')}_{season}_min-intr-points-{minimum_intrusion_points_pcnt}_sahara-{sahara_region[0]}.{sahara_region[1]}.{sahara_region[2]}.{sahara_region[3]}-roll.{sahara_clim_roll}-thresh_ta{ta_levels}_gh{gh_levels}.nc")
        dsplot.plot(norm=norm20, cmap=cmap20, add_colorbar=False)
        ax.coastlines(alpha=0.7)
        logger.info(f"cond NO intru {counts_station}")
        ax.scatter(
                    [float(i.split(":")[-1])for i in pX_days_station.location.values],
                    [float(i.split(":")[0])for i in pX_days_station.location.values],
                    c=counts_station,
                    s=45,
                    edgecolor="black",
                    linewidth=0.8,
                    cmap=cmap20,
                    norm=norm20,
                    zorder=100,
                )
        ax.set_extent(extent)
        area_of_impact = xr.open_dataarray(os.path.join(current_dir,f"files/postpro_files/area-of-impact-n{cluster_n+1}_clusters-k{k}_{'.'.join(cluster_vars)}_{ilon}.{flon}.{ilat}.{flat}_intrusions-clim-dates{identifier}_{period_cluster.replace('-','').replace('.','-')}_{season}_min-intr-points-{minimum_intrusion_points_pcnt}_sahara-{sahara_region[0]}.{sahara_region[1]}.{sahara_region[2]}.{sahara_region[3]}-roll.{sahara_clim_roll}-thresh_ta{ta_levels}_gh{gh_levels}.nc"))
        ax = plot_area_of_impact(ax, area_of_impact, "solid")
        ax.set_title(f"{season.upper()} intrusion type {cluster_n+1}\n{cluster_type_time.shape[0]} intrusion days")
        percent_of_intrusion_days_coinciding_withpX.append(pX_days[0].copy(data=counts))
        if cluster_n*row_id == 9:
            # cbar_ax = fig.add_axes([0.3, 0.03, 0.4, 0.02])  # Adjust the position [left, bottom, width, height]
            # cbar = plt.colorbar(plt.cm.ScalarMappable(norm=normn, cmap=cmapn), ticks=levelsn, spacing='uniform', cax=cbar_ax, orientation='horizontal', label=r"P(TX90p $\mid$ no intrusion day)")# - 10% (%)")
            # Add a new axis for the colorbar
            cbar_ax = fig.add_axes([0.3, 0.03, 0.4, 0.02])  # Adjust the position [left, bottom, width, height]
            cbar = plt.colorbar(plt.cm.ScalarMappable(norm=norm20, cmap=cmap20), ticks=levels20, spacing='uniform', cax=cbar_ax, orientation='horizontal', label=label)

# P(extreme t|intrusion)
def plot_pX_cond_intrusion(cluster_vars, pX_days, pX_days_station, k, period, period_cluster, bounds, minimum_intrusion_points_pcnt, identifier, season, row_id, fig, gs, sahara_clim_roll, sahara_region, percentile, ta_levels, gh_levels):
    ilon,flon,ilat,flat = bounds
    ss,es=period.split(".")
    sy,sm,sd = ss.split("-")
    ey,em,ed = es.split("-")
    clusters_vector_file = f"{current_dir}/files/postpro_files/clusters-k{k}_{'.'.join(cluster_vars)}_{ilon}.{flon}.{ilat}.{flat}_intrusions-clim-dates{identifier}_{period_cluster.replace('-','').replace('.','-')}_{season}_min-intr-points-{minimum_intrusion_points_pcnt}_sahara-{sahara_region[0]}.{sahara_region[1]}.{sahara_region[2]}.{sahara_region[3]}-roll.{sahara_clim_roll}-thresh_ta{ta_levels}_gh{gh_levels}.nc"
    clusters = xr.open_dataarray(clusters_vector_file).sel(time=slice(sy,ey))

    if str(percentile) == "90":
        label=r"P(TX90p $\mid$ intrusion day) (%)"
    elif str(percentile) == "99":
        label=r"P(TX99p $\mid$ intrusion day) (%)"

    percent_of_intrusion_days_coinciding_withpX = []
    # fig = plt.figure(figsize=(23, 5))
    # gs = matplotlib.gridspec.GridSpec(1, 4, figure=fig)
    for cluster_n in range(k):
        cluster_type_time = clusters.where(clusters==cluster_n, drop=True).time
        dates=pX_days.time
        is_in_dates = np.isin(dates, cluster_type_time)
        counts=np.count_nonzero(~np.isnan(pX_days[is_in_dates,:,:]),axis=0)/cluster_type_time.shape[0]*100
        dates_station=pX_days_station.time
        is_in_dates = np.isin(dates_station, cluster_type_time)
        counts_station = np.count_nonzero(~np.isnan(pX_days_station[is_in_dates,:]),axis=0)/cluster_type_time.shape[0]*100
        coord = {"Cluster_N": ("time",np.ones(pX_days.shape[0])+cluster_n)}
        logger.info(f"intrusion days for cluster {cluster_n+1}: {cluster_type_time.shape[0]}")
        ax = fig.add_subplot(gs[row_id, cluster_n], projection=ccrs.PlateCarree())
        dsplot = pX_days.drop_vars(["season_year"]).assign_coords(coord)[0].drop_vars(["time"]).copy(data=counts).rename(f"percentage_of_p{percentile}_days_coinciding_with_intrusion")
        dsplot.to_netcdf(f"{current_dir}/files/p{percentile}/p{percentile}-cond-intrusion_n{cluster_n}_clusters-k{k}_{'.'.join(cluster_vars)}_{ilon}.{flon}.{ilat}.{flat}_intrusions-clim-dates{identifier}_{period_cluster.replace('-','').replace('.','-')}_{season}_min-intr-points-{minimum_intrusion_points_pcnt}_sahara-{sahara_region[0]}.{sahara_region[1]}.{sahara_region[2]}.{sahara_region[3]}-roll.{sahara_clim_roll}-thresh_ta{ta_levels}_gh{gh_levels}.nc")
        station_nc = pX_days_station[0]
        station_nc.values = counts_station
        station_nc.to_netcdf(f"{current_dir}/files/p{percentile}/p{percentile}-cond-intrusion_n{cluster_n}_station_clusters-k{k}_{'.'.join(cluster_vars)}_{ilon}.{flon}.{ilat}.{flat}_intrusions-clim-dates{identifier}_{period_cluster.replace('-','').replace('.','-')}_{season}_min-intr-points-{minimum_intrusion_points_pcnt}_sahara-{sahara_region[0]}.{sahara_region[1]}.{sahara_region[2]}.{sahara_region[3]}-roll.{sahara_clim_roll}-thresh_ta{ta_levels}_gh{gh_levels}.nc")
        img = dsplot.plot(norm=norm100, cmap=custom_cmap100, add_colorbar=False)
        ax.coastlines(alpha=0.7)
        logger.info(f"cond intru {counts_station}")
        ax.scatter(
                    [float(i.split(":")[-1])for i in pX_days_station.location.values],
                    [float(i.split(":")[0])for i in pX_days_station.location.values],
                    c=counts_station,
                    s=45,
                    cmap=custom_cmap100,
                    edgecolor="black",
                    linewidth=0.8,
                    norm=norm100,
                    zorder=100,
                )
        ax.set_extent(extent)
        area_of_impact = xr.open_dataarray(os.path.join(current_dir,f"files/postpro_files/area-of-impact-n{cluster_n+1}_clusters-k{k}_{'.'.join(cluster_vars)}_{ilon}.{flon}.{ilat}.{flat}_intrusions-clim-dates{identifier}_{period_cluster.replace('-','').replace('.','-')}_{season}_min-intr-points-{minimum_intrusion_points_pcnt}_sahara-{sahara_region[0]}.{sahara_region[1]}.{sahara_region[2]}.{sahara_region[3]}-roll.{sahara_clim_roll}-thresh_ta{ta_levels}_gh{gh_levels}.nc"))
        ax = plot_area_of_impact(ax, area_of_impact, "solid")
        ax.set_title(f"{season.upper()} intrusion type {cluster_n+1}\n{cluster_type_time.shape[0]} intrusion days")
        percent_of_intrusion_days_coinciding_withpX.append(pX_days[0].copy(data=counts))
        if cluster_n*row_id == 9:
            # Add a new axis for the colorbar
            cbar_ax = fig.add_axes([0.3, 0.03, 0.4, 0.02])  # Adjust the position [left, bottom, width, height]
            # Add the colorbar to the new axis
            cbar = plt.colorbar(plt.cm.ScalarMappable(norm=norm100, cmap=custom_cmap100), ticks=levels100, spacing='uniform', cax=cbar_ax, orientation='horizontal', label=label)
            # plt.colorbar(img, cax=cbar_ax, orientation='horizontal', label="percentage of intrusion days that are pX days*****")

def plot_pX_cond_intrusion_lagged(cluster_vars, pX_days, k, period, period_cluster, bounds, minimum_intrusion_points_pcnt, identifier, season, row_id, fig, gs, sahara_clim_roll, sahara_region, lag, percentile, ta_levels, gh_levels):
    ilon,flon,ilat,flat = bounds
    ss,es=period.split(".")
    sy,sm,sd = ss.split("-")
    ey,em,ed = es.split("-")
    clusters_vector_file = f"{current_dir}/files/postpro_files/clusters-k{k}_{'.'.join(cluster_vars)}_{ilon}.{flon}.{ilat}.{flat}_intrusions-clim-dates{identifier}_{period_cluster.replace('-','').replace('.','-')}_{season}_min-intr-points-{minimum_intrusion_points_pcnt}_sahara-{sahara_region[0]}.{sahara_region[1]}.{sahara_region[2]}.{sahara_region[3]}-roll.{sahara_clim_roll}-thresh_ta{ta_levels}_gh{gh_levels}.nc"
    clusters = xr.open_dataarray(clusters_vector_file).sel(time=slice(sy,ey))

    dates = clusters.time.values
    # Step 1: Convert dates to pandas datetime
    dates = pd.to_datetime(dates)

    diff = pd.to_datetime(np.array(list(dates)+["2100-12-31"]))-pd.to_datetime(np.array(["1900-01-01"]+list(dates)))
    # Step 2: Identify contiguous dates
    diff = diff[1:]  # Compute the difference between consecutive dates
    contiguous_end_dates = dates[(diff != pd.Timedelta(days=1)) & (~diff.isnull())]  # Shift and filter the end of each contiguous period
    # Step 3: Add one day to each identified end of period
    filtered_for_length = []
    for date in contiguous_end_dates:
        if date-pd.Timedelta(days=1) in dates and date-pd.Timedelta(days=2) in dates:
            filtered_for_length.append(date)
    filtered_for_length = pd.to_datetime(filtered_for_length)
    extended_dates = contiguous_end_dates + pd.Timedelta(days=lag)
    # extended_dates = filtered_for_length + pd.Timedelta(days=lag)

    levels = np.linspace(0, 100, 11)  # 10 intervals from 0 to 100
    # Create a discrete colormap with BoundaryNorm
    cmap = plt.get_cmap("Oranges", len(levels) - 1)  # Colormap with 10 discrete colors
    norm = BoundaryNorm(levels, ncolors=cmap.N, clip=True)

    percent_of_intrusion_days_coinciding_withpX = []
    # fig = plt.figure(figsize=(23, 5))
    # gs = matplotlib.gridspec.GridSpec(1, 4, figure=fig)
    for cluster_n in range(k):
        cluster_type_time = clusters.where(clusters==cluster_n, drop=True).time
        # available_dates = [id for id, i in enumerate(filtered_for_length) if i in pd.to_datetime(cluster_type_time)]
        available_dates = [id for id, i in enumerate(contiguous_end_dates) if i in pd.to_datetime(cluster_type_time)]
        dates_to_select = extended_dates[available_dates]
        dates=pX_days.time.values
        is_in_dates = np.isin(dates, [f"{i.year:04}-{i.month:02}-{i.day:02}" for i in dates_to_select])
        counts=np.count_nonzero(~np.isnan(pX_days[is_in_dates,:,:]),axis=0)/dates_to_select.shape[0]*100
        coord = {"Cluster N": ("time",np.ones(pX_days.shape[0])+cluster_n)}
        logger.info(f"intrusion days for cluster {cluster_n+1}: {dates_to_select.shape[0]}")
        ax = fig.add_subplot(gs[row_id, cluster_n], projection=ccrs.PlateCarree())
        dsplot = pX_days.drop_vars(["season_year"]).assign_coords(coord)[0].drop_vars(["time"]).copy(data=counts).rename("percentage of pX days coinciding with intrusion")
        dsplot.to_netcdf(f"{current_dir}/files/p{percentile}/lag{lag}_p{percentile}-cond-intrusion_n{cluster_n}_clusters-k{k}_{'.'.join(cluster_vars)}_{ilon}.{flon}.{ilat}.{flat}_intrusions-clim-dates{identifier}_{period_cluster.replace('-','').replace('.','-')}_{season}_min-intr-points-{minimum_intrusion_points_pcnt}_sahara-{sahara_region[0]}.{sahara_region[1]}.{sahara_region[2]}.{sahara_region[3]}-roll.{sahara_clim_roll}-thresh_ta{ta_levels}_gh{gh_levels}.nc")
        img = dsplot.plot(norm=norm100, cmap=cmap100, add_colorbar=False)
        ax.coastlines(alpha=0.7)
        ax.set_extent(extent)
        area_of_impact = xr.open_dataarray(os.path.join(current_dir,f"files/postpro_files/area-of-impact-n{cluster_n+1}_clusters-k{k}_{'.'.join(cluster_vars)}_{ilon}.{flon}.{ilat}.{flat}_intrusions-clim-dates{identifier}_{period_cluster.replace('-','').replace('.','-')}_{season}_min-intr-points-{minimum_intrusion_points_pcnt}_sahara-{sahara_region[0]}.{sahara_region[1]}.{sahara_region[2]}.{sahara_region[3]}-roll.{sahara_clim_roll}-thresh_ta{ta_levels}_gh{gh_levels}.nc"))
        ax = plot_area_of_impact(ax, area_of_impact, "solid")
        ax.set_title(f"{season.upper()} intrusion type {cluster_n+1}\n{dates_to_select.shape[0]} post-intrusion days")
        percent_of_intrusion_days_coinciding_withpX.append(pX_days[0].copy(data=counts))
        if cluster_n*row_id == 9:
            # Add a new axis for the colorbar
            cbar_ax = fig.add_axes([0.3, 0.03, 0.4, 0.02])  # Adjust the position [left, bottom, width, height]
            # Add the colorbar to the new axis
            cbar = plt.colorbar(plt.cm.ScalarMappable(norm=norm100, cmap=cmap100), ticks=levels100, spacing='uniform', cax=cbar_ax, orientation='horizontal', label=r"P(TX90p $\mid$ day after intrusion) (%)")
    


# P(extreme t|intrusion)
def plot_compare(cluster_vars, k, period, period_cluster, bounds, minimum_intrusion_points_pcnt, identifier, season, row_id, fig, gs, sahara_clim_roll, sahara_region, percentile, ta_levels, gh_levels):
    ilon,flon,ilat,flat = bounds
    ss,es=period.split(".")
    sy,sm,sd = ss.split("-")
    ey,em,ed = es.split("-")
    clusters_vector_file = f"{current_dir}/files/postpro_files/clusters-k{k}_{'.'.join(cluster_vars)}_{ilon}.{flon}.{ilat}.{flat}_intrusions-clim-dates{identifier}_{period_cluster.replace('-','').replace('.','-')}_{season}_min-intr-points-{minimum_intrusion_points_pcnt}_sahara-{sahara_region[0]}.{sahara_region[1]}.{sahara_region[2]}.{sahara_region[3]}-roll.{sahara_clim_roll}-thresh_ta{ta_levels}_gh{gh_levels}.nc"
    clusters = xr.open_dataarray(clusters_vector_file).sel(time=slice(sy,ey))
    if str(percentile) == "90":
        label=r"P(TX90p $\mid$ IT\textit{n} intrusion day) / P(TX90p $\mid$ no IT\textit{n} intrusion day)"
    elif str(percentile) == "99":
        label=r"P(TX99p $\mid$ IT\textit{n} intrusion day) / P(TX99p $\mid$ no IT\textit{n} intrusion day)"
    for cluster_n in range(k):
        cluster_type_time = clusters.where(clusters==cluster_n, drop=True).time
        logger.info(f"intrusion days for cluster {cluster_n+1}: {cluster_type_time.shape[0]}")
        ax = fig.add_subplot(gs[row_id, cluster_n], projection=ccrs.PlateCarree())
        pXnointru = xr.open_dataarray(f"{current_dir}/files/p{percentile}/p{percentile}-cond-nointrusion_n{cluster_n}_clusters-k{k}_{'.'.join(cluster_vars)}_{ilon}.{flon}.{ilat}.{flat}_intrusions-clim-dates{identifier}_{period_cluster.replace('-','').replace('.','-')}_{season}_min-intr-points-{minimum_intrusion_points_pcnt}_sahara-{sahara_region[0]}.{sahara_region[1]}.{sahara_region[2]}.{sahara_region[3]}-roll.{sahara_clim_roll}-thresh_ta{ta_levels}_gh{gh_levels}.nc")
        pXintru = xr.open_dataarray(f"{current_dir}/files/p{percentile}/p{percentile}-cond-intrusion_n{cluster_n}_clusters-k{k}_{'.'.join(cluster_vars)}_{ilon}.{flon}.{ilat}.{flat}_intrusions-clim-dates{identifier}_{period_cluster.replace('-','').replace('.','-')}_{season}_min-intr-points-{minimum_intrusion_points_pcnt}_sahara-{sahara_region[0]}.{sahara_region[1]}.{sahara_region[2]}.{sahara_region[3]}-roll.{sahara_clim_roll}-thresh_ta{ta_levels}_gh{gh_levels}.nc")
        dsplot = pXintru/pXnointru
        pXnointru_station = xr.open_dataarray(f"{current_dir}/files/p{percentile}/p{percentile}-cond-nointrusion_n{cluster_n}_station_clusters-k{k}_{'.'.join(cluster_vars)}_{ilon}.{flon}.{ilat}.{flat}_intrusions-clim-dates{identifier}_{period_cluster.replace('-','').replace('.','-')}_{season}_min-intr-points-{minimum_intrusion_points_pcnt}_sahara-{sahara_region[0]}.{sahara_region[1]}.{sahara_region[2]}.{sahara_region[3]}-roll.{sahara_clim_roll}-thresh_ta{ta_levels}_gh{gh_levels}.nc")
        pXintru_station = xr.open_dataarray(f"{current_dir}/files/p{percentile}/p{percentile}-cond-intrusion_n{cluster_n}_station_clusters-k{k}_{'.'.join(cluster_vars)}_{ilon}.{flon}.{ilat}.{flat}_intrusions-clim-dates{identifier}_{period_cluster.replace('-','').replace('.','-')}_{season}_min-intr-points-{minimum_intrusion_points_pcnt}_sahara-{sahara_region[0]}.{sahara_region[1]}.{sahara_region[2]}.{sahara_region[3]}-roll.{sahara_clim_roll}-thresh_ta{ta_levels}_gh{gh_levels}.nc")
        dsplot_station = pXintru_station/pXnointru_station
        img = dsplot.plot(levels=levelsnL, cmap=new_cmapnL, norm=normnL, add_colorbar=False)
        ax.coastlines(alpha=0.7)
        ax.set_extent(extent)
        ax.scatter(
            [float(i.split(":")[-1])for i in dsplot_station.location.values],
            [float(i.split(":")[0])for i in dsplot_station.location.values],
            c=dsplot_station.values,
            s=45,  
            cmap=new_cmapnL,
            edgecolor="black",  
            linewidth=0.8,
            norm=normnL,
            zorder=100,
        )
        area_of_impact = xr.open_dataarray(os.path.join(current_dir,f"files/postpro_files/area-of-impact-n{cluster_n+1}_clusters-k{k}_{'.'.join(cluster_vars)}_{ilon}.{flon}.{ilat}.{flat}_intrusions-clim-dates{identifier}_{period_cluster.replace('-','').replace('.','-')}_{season}_min-intr-points-{minimum_intrusion_points_pcnt}_sahara-{sahara_region[0]}.{sahara_region[1]}.{sahara_region[2]}.{sahara_region[3]}-roll.{sahara_clim_roll}-thresh_ta{ta_levels}_gh{gh_levels}.nc"))
        ax = plot_area_of_impact(ax, area_of_impact, "solid")
        ax.set_title(f"{season.upper()} intrusion type {cluster_n+1}\n{cluster_type_time.shape[0]} intrusion days")
        logger.info(cluster_n*row_id)
        if cluster_n*row_id == 9:
            # Add a new axis for the colorbar
            cbar_ax = fig.add_axes([0.2, 0.03, 0.6, 0.02])  # Adjust the position [left, bottom, width, height]
            ticks=[f"{i:.01f}" for i in levelsnL[:10]]+[int(i) for i in levelsnL[10:-1]]+[""]
            logger.info(ticks)
            cbar = fig.colorbar(plt.cm.ScalarMappable(norm=normnL, cmap=new_cmapnL), extend="max", ticks=levelsnL, spacing='uniform', cax=cbar_ax, orientation='horizontal', label=label)
            cbar.ax.set_xticklabels(ticks)


def compute_pX(period, X, overwrite=False):
    """
    Find the tasmax pX percentile days
    """
    file_stations = os.path.join(current_dir, "files", "stations.nc")
    file_era = os.path.join(current_dir, "files", "tasmax_1959-2022_01-12.nc")
    for dst, file in zip(["","station-"],[file_era, file_stations]):
        if os.path.isfile(file) and not overwrite:
            logger.info(f"skipping copute_pX of already existing {file}")
            continue
        tas_daily_max = xr.open_dataarray(file)
        ss,es=period.split(".")
        sy,sm,sd = ss.split("-")
        ey,em,ed = es.split("-")
        tas_daily_max = xr.DataArray.from_iris(extract_time(tas_daily_max.to_iris(),sy,sm,sd,ey,em,ed))
        # Apply a rolling window of 7 days centered (3 days before and after each day)
        tas_rolling = tas_daily_max.rolling(time=7, center=True).construct('window_dim')
        # Extract day of the year (doy)
        doy = tas_rolling['time.dayofyear']
        # Group by day of the year
        tas_grouped = tas_rolling.groupby(doy)
        # Compute the Xth percentile for each group
        tas_Xth_percentile = tas_grouped.quantile(X/100, dim=['time', 'window_dim'])
        # Assign coordinates to the result
        tas_Xth_percentile = tas_Xth_percentile.rename({'dayofyear': 'time'})
        tas_Xth_percentile = tas_Xth_percentile.to_iris()
        stacked_clim = []
        dts=tas_daily_max.to_iris().coord("time").units.num2date(tas_daily_max.to_iris().coord("time").points)
        for year in np.arange(int(dts[0].year), int(dts[-1].year)+1):
            clim_data = list(tas_Xth_percentile.data)
            if extract_time(tas_daily_max.to_iris(), year, 1, 1, year+1, 1, 1).shape[0] != 366:
                # Add 29th of February
                del clim_data[59]
            stacked_clim += clim_data
        clim_cube = tas_daily_max.to_iris().copy(np.array(stacked_clim))
        iris.save(clim_cube, os.path.join(current_dir, "files", "rev", f"{dst}tasmax-p{X}-7dayrolling_{period}.nc"))
